//=========================================================================
//
//	FILE:				CP_KeyInputFilters.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Classes to contol the input of text into edit fields.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 222 $
//						Latest Checkout:  $Date: 2005-06-28 17:17:59 -0700 (Tue, 28 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_KeyInputFilters
#define _H_CP_KeyInputFilters
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_M_ReferenceCounted.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

enum CP_CharStatus
{
	CP_CharStatus_Input,			/*!< Input the character */
	CP_CharStatus_Delete,			/*!< Delete key */
	CP_CharStatus_Cursor,			/*!< Cursor key */
	CP_CharStatus_Ignore,			/*!< Ignore the key */
	CP_CharStatus_PassUp,			/*!< Pass up to the next handler */
	CP_CharStatus_Scroll			/*!< Scroll key */
};

//=======================================================================
// CP_KeyInputFilter - Base Class
//		Don't create an instance of this.
//=======================================================================
//!  CP_KeyInputFilter
/*!
*	A CP_M_ReferenceCounted class to filter keystrokes in edit fields.
*/
class CP_EXPORT CP_KeyInputFilter : public CP_M_RefCounted
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Destructor.
	virtual								~CP_KeyInputFilter();
	//@}

	//! @name Validation
	//@{
										/*!
										*	Subclasses must override this to determine if the character passed in is valid.
										*
										*	@param		inChar				The character to check.
										*	
										*	@exception	none
										*	
										*	@return		CP_CharStatus		Type of character passed in.
										*/
	virtual CP_CharStatus				IsKeyValid( char inChar ) = 0;
	//@}

protected:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_KeyInputFilter();
	//@}

	//! @name Testers
	//@{
			bool						IsADeleteKey( char inChar );
			bool						IsACursorKey( char inChar );
			bool						IsAScrollKey( char inChar );
	//@}

	// class data
protected:

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Validation
	//@{
										/*!
										*	Subclasses override this to determine if the character passed in is valid.
										*	This version checks the passed in Unicode character.
										*
										*	@param		inChar				The Unicode character to check.
										*	@param		inDataLength		Length of the passed in string.
										*	
										*	@exception	none
										*	
										*	@return		CP_CharStatus		Returns CP_CharStatus_PassUp.
										*/
		virtual CP_CharStatus			IsKeyValid(
												const CPUniChar */*inChar*/,
												UInt32 /*inDataLength*/ )			{ return( CP_CharStatus_PassUp ); }
	//@}
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

//=======================================================================
// CP_KeyInputFilter_AlphaNumeric
//		Only allows alpha numeric input characters.
//=======================================================================
/*!
*	A CP_KeyInputFilter class to allow alpha numeric keystrokes.
*/
class CP_EXPORT CP_KeyInputFilter_AlphaNumeric : public CP_KeyInputFilter
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_KeyInputFilter_AlphaNumeric( bool inAllowSpace );
	//! Destructor.
	virtual								~CP_KeyInputFilter_AlphaNumeric();
	//@}

	//! @name Validation
	//@{
	virtual CP_CharStatus				IsKeyValid( char inChar );
	//@}

	// class data
protected:
	bool								fAllowSpace;	/*! Flag to indicate if space is considered allowable. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

//=======================================================================
// CP_KeyInputFilter_Numeric
//		Only allows numeric input characters.
//=======================================================================
/*!
*	A CP_KeyInputFilter class to allow numeric keystrokes.
*/
class CP_EXPORT CP_KeyInputFilter_Numeric : public CP_KeyInputFilter
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_KeyInputFilter_Numeric();
	//! Destructor.
	virtual								~CP_KeyInputFilter_Numeric();
	//@}

	//! @name Validation
	//@{
	virtual CP_CharStatus				IsKeyValid( char inChar );
	//@}

	// class data
protected:

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Validation
	//@{
		virtual	CP_CharStatus			IsKeyValid(
												const CPUniChar *inChar,
												UInt32 inDataLength );
	//@}
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

class CP_EXPORT CP_KeyInputFilter_UnicodeAlphaNumeric : public CP_KeyInputFilter
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_KeyInputFilter_UnicodeAlphaNumeric( bool inAllowSpace );
	//! Destructor.
	virtual								~CP_KeyInputFilter_UnicodeAlphaNumeric();
	//@}

	//! @name Validation
	//@{
	virtual CP_CharStatus				IsKeyValid( char inChar );
	//@}

	// class data
protected:
								
	bool								fAllowSpace;	/*! Flag to indicate if space is considered allowable. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Validation
	//@{
		virtual	CP_CharStatus			IsKeyValid(
												const CPUniChar *inChar,
												UInt32 inDataLength );
	//@}
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_KeyInputFilters
