//=========================================================================
//
//	FILE:				CP_Mutex.h
//	
//	PURPOSE:			Class for hiding thread mutex platform implementations.
//	
//	HISTORY:			1st Version - January 2005.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 159 $
//						Latest Checkout:  $Date: 2005-04-27 15:56:44 -0700 (Wed, 27 Apr 2005) $
//
//=========================================================================

#ifndef _H_CP_Mutex
#define _H_CP_Mutex
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if CP_ThreadingAPI_PThread
	#if TARGET_OS_MAC
		#include <pthread.h>
	#endif
#endif


CPLAT_Begin_Namespace_CPLAT


/*!
    @class    CP_Mutex 
    @brief    Mutex use with preemptive threading.

	Mutex class that abstracts away different underlying threading APIs and
	allows a common access to thread synchronization.\n
	For preemptive threading implementations, the mutex will use the underlying
	threading APIs to setup, lock and unlock mutices, whereas for cooperative
	threading implementations, the mutices will do nothing when locked / unlocked,
	assuming that there is no concurrent execution happening anyway.
    
	@sa	CP_M_Thread
*/
class CP_Mutex : CP_NoCopy
{
public:
	/*!
	 *   @brief   Allocate the OS mutex resource.
	 *
	 *   @exception std::runtime_error		raised if mutex resource could not be allocated.
	 *
	 */
	CP_Mutex();
	/*!
	 *   @brief   Deallocate the OS mutex resource.
	 *
	 *   @exception none.
	 *
	 */
	~CP_Mutex();
	/*!
	 *   @brief   Lock the mutex.
	 *
	 *   Attempts to lock the mutex, blocking until the lock operation succeeded.
	 *
	 *   @exception std::runtime_error		raised if lock could not be acquired.
	 *
	 */
	void lock();
	/*!
	 *   @brief   Unlock the mutex.
	 *
	 *   Unlocks a previously locked mutex. When calling unlock for an unlocked mutex,
	 *	 the behavior is not specified.
	 *
	 *   @exception std::runtime_error		raised if unlock failed.
	 *
	 */
	void unlock();
private:
#if TARGET_OS_MAC
	#if TARGET_RT_MAC_MACHO && CP_ThreadingAPI_PThread
		typedef pthread_mutex_t mutex_type;
		mutex_type	fMutex_;			//!< the OS mutex resource
	#endif
	
	#if CP_ThreadingAPI_MPTask
		typedef MPCriticalRegionID mutex_type;
		mutex_type	fMutex_;			//!< the OS mutex resource
	#endif
#endif

#if TARGET_OS_WIN32
		typedef CRITICAL_SECTION mutex_type;
		mutex_type fMutex_;				//!< the OS mutex resource
#endif

#if TARGET_OS_LINUX
#endif
};


/*!
@class    CP_MutexLock 
@brief    Helper class for locking and an unlocking a mutex.

Locks the specified mutex in it's constructor and unlocks it again in the destructor.
Very helpful when working with exceptions and multiple return paths, since you won't
ever forget to unlock that mutex.
*/
class CP_MutexLock : public CP_NoCopy
{
public:
	explicit CP_MutexLock(CP_Mutex& m) : m_(m)					{ m_.lock(); }
	~CP_MutexLock()												{ m_.unlock(); }
private:
	CP_Mutex& m_;
};

//! Macro for locking a mutex using CP_MutexLock
#define CP_LOCK_MUTEX(lock, mutex) CPLAT::CP_MutexLock lock(mutex);


CPLAT_End_Namespace_CPLAT

#endif // _H_CP_Mutex

