//=========================================================================
//
//	FILE:				CP_Object.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Base class for objects used in CPLAT.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 245 $
//						Latest Checkout:  $Date: 2005-07-27 23:36:26 -0700 (Wed, 27 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_Object
#define _H_CP_Object
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_DataValueHolder.h"
#include "CP_M_CommandHandler.h"
#include "CP_NoCopy.h"
#include "CP_OSTypes.h"
#include "CP_SignalSlot.h"
#include "CP_SignalSlotHolder.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Object
*	@brief		Base class for objects used in CPLAT.
*
*				This base class provides lists for registering and finding CP_Signal1 and CP_Slot1
*				objects that can be identified by a CP_String name to provide a signal / slot
* 				connection between objects.
*
*	\begincode
*		CP_DataValueSignal			*signal = object->FindSignal( signalName );
*		CP_DataValueSlot			*slotConnection = object->FindSlot( slotName );
*
*		signal->connect( slotConnection );
*	\endcode
*/
class CP_EXPORT CP_Object : public CP_NoCopy
{
public:
											// Construction / destruction
										CP_Object();
	virtual								~CP_Object();
			
										/*!
										*	Sets the identifier for this object. If a specific identifier is not
										*	set with this function, then a default id is used, which is simply a
										*	running count of the number of objects created.
										*	
										*	@param	inIdentifier		The objects identifier.
										*	
										*	@return	void
										*/
			void						SetID( FourCharCode inIdentifier )							{ fObjectID = inIdentifier; }

										/*!
										*	Returns the objects identifier.
										*	
										*	@return	FourCharCode		The identifier of this object.
										*/
			FourCharCode				GetID()														{ return( fObjectID ); }
			
											// Signals
			void						RegisterSignal( const CP_SignalHolder& inSignal );
			CP_DataValueSignal			*FindSignal( const CP_String& inSignalName );
			UInt32						GetNumberSignals();
			CP_String					GetNthSignalName( UInt32 inSignalIndex );

											// Slots
			void						RegisterSlot( const CP_SlotHolder& inSlot );
			CP_DataValueSlot			FindSlot( const CP_String& inSlotName );
			UInt32						GetNumberSlots();
			CP_String					GetNthSlotName( UInt32 inSlotIndex );

											// Connects a command number sent by a control to this->handler
										template<class R, class P1, class P2>
										CP_CommandHandlerSignal *Connect(CPLAT::CP_CommandNo cmd, void (R::*handler)(P1, P2), SInt32 mode)
										{
											CP_CommandHandlerSignal				*signal = CP_new CP_CommandHandlerSignal();
											signal->connect( CP_SigSlot::slot((R*)this, handler) );

											CP_M_CommandHandler					*commandHandler = dynamic_cast<CP_M_CommandHandler *>((R*)this);
											if( commandHandler != NULL )
												commandHandler->RegisterCommand( cmd, signal, mode );
											
											return (CP_CommandHandlerSignal*)signal;
										}

										// connect a control signal (signalName) to void handler(CP_DataValueHolder&, OSStatus&) 
										template<class R, class P1, class P2>
										void Connect(CP_Object *control, const CPLAT::CP_String& signalName, void (R::*handler)(P1, P2))
										{
											CP_DataValueSignal				*dvs = control->FindSignal( signalName );
											
											if( dvs )
												dvs->connect( CP_SigSlot::slot((R*)this, handler) );
										}

	static	void						Connect(
												CP_Object *sender,
												const CP_String& signalName,
												CP_Object *receiver,
												const CP_String& slotName,
												const bool reciprocal = false );
	static void							Connect(
												CP_Object *sender,
												CP_Object *receiver, 
												const bool reciprocal = false,
												const CP_String& signalName = kCP_Signal_ValueChanged, 
												const CP_String& slotName = kCP_Slot_SetValue );

protected:

private:
											// Don't allow copy and assignment
										CP_Object( const CP_Object& );
	CP_Object&							operator = ( const CP_Object& );

	// class data
protected:
	CP_Signals_Array					*fSignals;		/*! List of registered signals for this object. */

	CP_Slots_Array						*fSlots;		/*! List of registered slots for this object. */
	
	FourCharCode						fObjectID;		/*! Object identifier. */

	static FourCharCode					fObjectIDCount;	/*! Running count of objects to use as default ids. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Object

