//=========================================================================
//
//	FILE:				CP_Graphics_Imp_GDI.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Windows GDI implementation of the CP_Graphics object.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 247 $
//						Latest Checkout:  $Date: 2005-07-30 19:06:38 -0700 (Sat, 30 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_Graphics_Imp_GDI
#define _H_CP_Graphics_Imp_GDI
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_WIN32

	// CPLAT_Framework
#include "CP_Graphics_Imp.h"

CPLAT_Begin_Namespace_CPLAT

enum CP_E_PathOperation
{
	CP_E_PathOperation_NoOp,
	CP_E_PathOperation_ClosePath,
	CP_E_PathOperation_MoveTo,
	CP_E_PathOperation_LineTo,
	CP_E_PathOperation_Rect,
	CP_E_PathOperation_Oval,
	CP_E_PathOperation_Bezier,
	CP_E_PathOperation_QuadBezier,
	CP_E_PathOperation_Poly
};

// ===================================================================================
//	CP_S_PathOperation - structure used to hold commands for constructing a path.
// ===================================================================================
struct CP_S_PathOperation
{
	CP_E_PathOperation		fPathOp;
	CP_Point				fStartPoint;
	CP_Point				fEndPoint;
	CP_Point				fControlPointOne;
	CP_Point				fControlPointTwo;
	
							CP_S_PathOperation();
							CP_S_PathOperation(
									CP_E_PathOperation inPathOp,
									CP_Point inStartPoint,
									CP_Point inEndPoint );
							CP_S_PathOperation(
									CP_E_PathOperation inPathOp,
									CP_Point inStartPoint,
									CP_Point inEndPoint,
									CP_Point inControlPointOne,
									CP_Point inControlPointTwo );
							CP_S_PathOperation( const CP_S_PathOperation& inOperation );

	CP_S_PathOperation&		operator =( const CP_S_PathOperation& inOperation );
};

	// list of path operations
typedef std::vector<CP_S_PathOperation, std::allocator<CP_S_PathOperation> >					CP_S_PathOperation_Array;
typedef std::vector<CP_S_PathOperation, std::allocator<CP_S_PathOperation> >::iterator			CP_S_PathOperation_Iterator;
typedef std::vector<CP_S_PathOperation, std::allocator<CP_S_PathOperation> >::reverse_iterator	CP_S_PathOperation_ReverseIterator;

/*!
*	@class		CP_Graphics_Imp_GDI
*	@brief		CP_Graphics_Imp subclass to perform drawing using Windows GDI.
*
*				This is the CP_Graphics_Imp class used by a CP_Graphics object when the
*				user has selected Windows GDI as the underlying graphics engine. This will
*				also be selected as the native platform type when not running under Windows
*				and GDI+ drawing is not available.
*/
class CP_EXPORT CP_Graphics_Imp_GDI : public CP_Graphics_Imp
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Graphics_Imp_GDI( CP_Canvas& inDestCanvas );
	//! Destructor.
	virtual								~CP_Graphics_Imp_GDI();
	//@}

	//! @name Setters
	//@{
	virtual	void						SetAlpha( float inAlpha );
	virtual	void						SetLineWidth( float lineWidth );
	virtual	void						SetStrokeColor( CP_ARGBColor& inStrokeColor );
	virtual void						SetFillColor( CP_ARGBColor& inFillColor );
	//@}
			
	//! @name Bezier
	//@{
	virtual	void						StrokeBezier(
											const CP_Point& inStartPoint,
											const CP_Point& inControlPointOne,
											const CP_Point& inControlPointTwo,
											const CP_Point& inEndPoint );
	virtual	void						StrokeQuadBezier(
											const CP_Point& inStartPoint,
											const CP_Point& inControlPointOne,
											const CP_Point& inEndPoint );
	//@}

	//! @name Gradient
	//@{
	virtual	void						DrawGradient(
												CP_E_GradientType inGradientType,
												CP_Rect& inDestBounds );
	virtual	void						SetGradientAngle( SInt16 inAngle )						{ fGradientAngle = inAngle; }
	virtual	void						SetGradientColor(
												const SInt16 inIndex,
												const CP_ARGBColor& inColour );
	virtual	void						SetGradientCount( SInt16 inGradientCount )				{ fGradientCount = inGradientCount; }
	//@}

	//! @name Icons
	//@{
			void						DrawIcon(
												const CP_Rect& inBounds,
												CP_E_IconStyle inIconType,
												SInt16 inIconID );
			void						DrawIcon(
												CP_OSIcon& inPicture,
												const CP_Rect& inBounds );
	//@}

	//! @name Images
	//@{
	virtual	void						DrawImage(
												const CP_FileSystemSpec& inImageSpec,
												CP_Rect& inDestBounds );
	virtual	void						DrawImage(
												const CP_FileSystemSpec& inImageSpec,
												CP_Rect& inSourceBounds,
												CP_Rect& inDestBounds );
	virtual	void								DrawPicture(
												CP_OSPicture& inPicture,
												const CP_Rect& inBounds );
	//@}

	//! @name Lines
	//@{
	virtual	void						Line(
												const CP_Point& inPoint,
												const CP_Size& inSize );
	virtual	void						Line(
												float inStartHorLocation,
												float inStartVerLocation,
												float inEndHorLocation,
												float inEndVerLocation );
	//@}

	//! @name Paths
	//@{
	virtual	void						BeginPath();
	virtual	void						ClosePath();
	virtual	void						StrokePath();
	virtual	void						FillPath();
	virtual	void						FillPath( CP_Pattern& inPattern );
	virtual	void						PathMoveTo( const CP_Point& inPoint );
	virtual	void						PathMoveTo(
												float inHorLocation,
												float inVerLocation );
	virtual	void						PathAddLineTo( const CP_Point& inPoint );
	virtual	void						PathAddLines(
												const CP_Point inPoint[],
												SInt16 inNumberPoints );
	virtual	void						PathAddLine(
												const CP_Point& inStartPoint,
												const CP_Point& inEndPoint );
	virtual	void						PathAddLine(
												float inStartH,
												float inStartV,
												float inEndH,
												float inEndV );
	virtual	void						PathAddRect( const CP_Rect& inBounds );
	virtual	void						PathAddOval( const CP_Rect& inBounds );
	virtual	void						PathAddBezier(
											const CP_Point& inStartPoint,
											const CP_Point& inControlPointOne,
											const CP_Point& inControlPointTwo,
											const CP_Point& inEndPoint );
	virtual	void						PathAddQuadBezier(
											const CP_Point& inStartPoint,
											const CP_Point& inControlPointOne,
											const CP_Point& inEndPoint );
	virtual	void						PathAddPoly( CP_Point_Array& inPoints );
	virtual	void						ClipPath();
	//@}

	//! @name Polygons
	//@{
	virtual	void						StrokePoly( CP_Point_Array& inPoints );
	virtual	void						FillPoly( CP_Point_Array& inPoints );
	//@}

	//! @name Rects
	//@{
	virtual	void						StrokeRect( const CP_Rect& inBounds );
	virtual	void						EraseRect( const CP_Rect& inBounds );
	virtual	void						FillRect( const CP_Rect& inBounds );
	//@}

	//! @name Round Rects
	//@{
	virtual	void						StrokeRoundRect(
												const CP_Rect& inBounds,
												SInt16 inHorRadius,
												SInt16 inVerRadius );
	virtual	void						FillRoundRect( 
												const CP_Rect& inBounds,
												SInt16 inHorRadius,
												SInt16 inVerRadius );
	//@}

	//! @name Oval
	//@{
	virtual	void						StrokeOval( const CP_Rect& inBounds );
	virtual	void						FillOval( const CP_Rect& inBounds );
	//@}

	//! @name Text
	//@{
	virtual	void						DrawString(
												CP_FontSpec& inFontSpec,
												const CP_String& inText,
												const CP_Point& inStartPoint );
	virtual	void						DrawText(
												CP_FontSpec& inFontSpec,
												const CP_String& inText,
												const CP_Rect& inBounds,
												SInt16 inHorAlign = CP_E_TextAlign_HorLeft,
												SInt16 inVerAlign = CP_E_TextAlign_VerTop,
												bool inWrapText = true );
	virtual	void						DrawTrunc(
												CP_FontSpec& inFontSpec,
												const CP_String& inText,
												const CP_Rect& inBounds,
												SInt16 inHorAlign = CP_E_TextAlign_HorLeft,
												SInt16 inVerAlign = CP_E_TextAlign_VerTop );
	//@}

protected:

private:
	//! @name Misc
	//@{
			void						FrameQuadCurve(
												CP_Canvas& inDestCanvas,
												const CP_Point& inStartPoint,
												const CP_Point& inControlPointOne,
												const CP_Point& inEndPoint );
			void						FrameCurve(
												CP_Canvas& inDestCanvas,
												kCP_QuadCurveCurve *inQuadCurve,
												SInt16 inLevel );

			CP_Region					CreatePathRegion();

			CP_Rect						GetPolyBounds();

			void						InitGradient();
	//@}

	// class data
protected:
	bool								fNeedTransform;			/*! Flag to indicate if any drawing operation needs transformation. */
	CP_Region							fClipPath;				/*! Region used for clipping. */
	
	CP_S_PathOperation_Array			fPath;					/*! An array of path operations. */

	CP_ARGBColor						fStrokeColor;			/*! Color used when stroking an area. */
	CP_ARGBColor						fFillColor;				/*! Color used when filling an area. */

	CP_Point_Array						fPoints;				/*! Points used to draw the polygon. */
	CP_Point_Array						fTempRegionPoints;		/*! Points used to draw a region. Working copy. */

	CP_E_GradientType					fGradientType;			/*! Type of gradient. */
	SInt16								fGradientCount;			/*! Number of colors in gradient. */
	CP_ARGBColor						fColors[ kCP_MaxGradientColors ];		/*! List of colors in gradient. */
	SInt16								fOffsets[ kCP_MaxGradientColors ];		/*! Offsets used to draw gradient. */
	SInt16								fGradientAngle;			/*! Angle used to draw gradient. */
};

CPLAT_End_Namespace_CPLAT

#endif	// __BUILD_WIN__

#endif	// _H_CP_Graphics_Imp_GDI
